--- HUD specific variables
hud = {
	live = { -- variables that keep track of "activated" objects such as open windows and clicked buttons
		openwindows = {
			-- these table valuesare build like this: {coords = {l, t, r, b, buttons = {(one for each button) = {l, t, r, b} }, grid = {} } }
				HudBar = {}
		},
		openmenus = {
				--Menu = {}
		},
	},

	windows = {},
}

--Classes
HudWindow_Class = class(function(tableholder, Name, Position, Images, Buttons, vars)
tableholder.Name = Name -- Identifier, internal use only
tableholder.Position = Position -- The position of hte window
tableholder.Images = Images	--All of the images used by the window
tableholder.Buttons = Buttons 	-- Keeps track of which buttons this window holds with values shown as a table: {Relative X, Relative Y, Action, {Images}}
tableholder.vars = vars 	-- A place to store any variables unique to that particular window
end)

-- Methods
function HudWindow_Class:draw( Xoffset, yoffset )
	--set the color to default, just in case
	love.graphics.setColor(255,255,255,255)


	if self == hud.windows.Menu then
		-- draw the sections of the window
		for eachsection in ipairs(self.vars.imagechunks) do
			love.graphics.draws(self.Images[1][3][1],
			self.vars.imagechunks[eachsection].cx + self.vars.imagechunks[1].w/2 + hud.windows.Menu.vars.location[1],
			self.vars.imagechunks[eachsection].cy + self.vars.imagechunks[1].h/2 + hud.windows.Menu.vars.location[2],
			self.vars.imagechunks[eachsection].x,
			self.vars.imagechunks[eachsection].y,
			self.vars.imagechunks[eachsection].w,
			self.vars.imagechunks[eachsection].h
			)
		end
		--draw text
		love.graphics.setFont(font.label)
		for eachentry in ipairs(self.vars.entries) do
			love.graphics.draw(
				self.vars.entries[eachentry],
				hud.windows.Menu.vars.location[1] + 7,
				hud.windows.Menu.vars.location[2] + 15 * eachentry + (0)
				)
		end
		return
	end

	-- draw backgrounds, of variable size
	for numdraw in pairs(self.Images) do
		love.graphics.draw(self.Images[numdraw][3][1],
		self.Position[1] + self.Images[numdraw][1],
		self.Position[2] + self.Images[numdraw][2])
	end
	--draw windows
	for numdraw in pairs(self.Images) do
		love.graphics.draw(self.Images[numdraw][3][1],
		self.Position[1] + self.Images[numdraw][1],
		self.Position[2] + self.Images[numdraw][2])
	end

	--draw buttons
	for b in pairs(self.Buttons) do
		for i in pairs(self.Buttons[b][4]) do
			if self.Buttons[b][4][1] then
				love.graphics.draw(self.Buttons[b][4][i],
				self.Position[1] + self.Buttons[b][1],
				self.Position[2] + self.Buttons[b][2])
			end
		end
	end

	-- draw text
	love.graphics.setColor(0,0,0,255)
	if self.vars.text then
		for t in pairs(self.vars.text) do
			love.graphics.printf( 		self.vars.text[t][1],
									self.Position[1] + self.vars.text[t][2],
									self.Position[2] + self.vars.text[t][3],
									self.vars.text[t][4],
									"left"
								)
		end
	end


	-- draw inventory specific things
	if self == hud.windows.Inventory then
			-- draw items in the inventory grid
		for x = 1, self.vars.grid.x do
			for y = 1, self.vars.grid.y do
				for cell, item in ipairs( player.inventory[x][y] ) do
					love.graphics.setFont( font.default )
					love.graphics.setColor(255,255,255,255)
					love.graphics.draw(
						player.inventory[x][y][1].hudimage,
						self.Position[1] + self.vars.grid.xorigin + ( (x - 1) * self.vars.grid.size),
						self.Position[2] + self.vars.grid.yorigin + ( (y - 1) * self.vars.grid.size),
						0,
						0.6
					)
				end
			end
		end

		-- draw item descriptions and available actions
		if self.vars.SelectedInventoryItem then
			-- image
			love.graphics.draw(
				self.vars.SelectedInventoryItem.hudimage,
				self.Position[1] + 35,
				self.Position[2] - 45,
				0,
				0.8
			)

			-- main title
			love.graphics.setFont( font.label )
			love.graphics.printf(
				self.vars.SelectedInventoryItem.name,
				self.Position[1] + 60,
				self.Position[2] - 45,
				80,
				"left"
			)
			love.graphics.setFont( font.default )

		end
	end

	-- draw Menus if necessary
end


-- Windows
hud.windows = {

	HudBar = HudWindow_Class(
	"Main Hud Bar",
	{0, 540},
	{	{0, 0, {images.hud.main.bar} }	},
	{ 	{250, 6, {"world_AdvanceDay"}, {images.hud.buttons.nextgen} },
		{10, 0, {"hud_ToggleWindow", "Inventory"}, {images.hud.buttons.inventory} },
		{60, 0, {"hud_ToggleWindow", "Examine"}, {images.hud.buttons.examine} },	},
	{}
	),

	Examine = HudWindow_Class(
	"Examine Window",
	{675, 400},
	{ 	{0, 0, {images.hud.windows.examine_window},}	},
	{ {-110, 37, {"hud_ToggleWindow", "Examine"}, {images.hud.buttons.close} }, },
	{}
	),

	Inventory = HudWindow_Class(
	"Inventory Window",
	{500, 0},
	{
		{0, 0, { images.hud.windows.inventory_window } },
		{7, 235, { images.hud.boxes.inventory_main } },
		{9, 237, { images.hud.boxes.inventory_grid } },
	},
	{
		{70, 10, {"hud_InventorySlotSelect", "head"}, {images.hud.buttons.inventory_slot} }, 	-- head slot
		{70, 55, {"hud_InventorySlotSelect", "torso"}, {images.hud.buttons.inventory_slot} }, 	-- torso slot
		{70, 100, {"hud_InventorySlotSelect", "belt"}, {images.hud.buttons.inventory_slot} }, 		-- belt slot
		{70, 145, {"hud_InventorySlotSelect", "legs"}, {images.hud.buttons.inventory_slot} }, 		-- pants slot
		{70, 190, {"hud_InventorySlotSelect", "feet"}, {images.hud.buttons.inventory_slot} },		-- shoes slot
		{25, 90, {"hud_InventorySlotSelect", "right_hand"}, {images.hud.buttons.inventory_slot} },	-- right hand slot
		{115, 90, {"hud_InventorySlotSelect", "left_hand"}, {images.hud.buttons.inventory_slot} }, 	-- left hand slot
		{20, 20, {"hud_InventorySlotSelect", "face_neck"}, {images.hud.buttons.inventory_slot} },	-- face/neck slot
		{120, 20, {"hud_InventorySlotSelect", "back"}, {images.hud.buttons.inventory_slot} }, 	-- back slot

		{7, 520, {"hud_ToggleWindow", "Inventory"}, {images.hud.buttons.close} },
	},
	{	SelectedSlot = nil,
		SelectedGriddCell = nil,
		SelectedInventoryItem = nil,
		text = {
			{"The Slot", 30, -250, 100},
			{"Weight/max", 30, -235, 150},
			{"Volume/max", 30, -220, 150},
			{"Conditions, including warmth, damage, disease, infection, exhaustion, current use, etc. This is brief, details in diff window/tab", 35, -205, 100},
		},
		grid = {x = 5, y = 9, size = 30, xorigin = 9, yorigin = 237, },
	}
	),

	Menu = HudWindow_Class(
	"Context Menu",
	{0, 0},
	{
		{0, 0, { images.hud.menus.menu_action} },
	},
	{},
	{
		build = true,
		entries = {},
		entryfontsize = 10,
		imagechunks = {}
	}
	),
}


function hud:load()

		hud:GetCurrentHudCoords()

end

function hud:update()
	--update inventory window images in each slot
	if hud.live.openwindows.Inventory then
		for id, slots in ipairs(player.inventory.slots) do
			if player.inventory[ slots ].topitem then
				hud.windows.Inventory.Buttons[ id ][4][2] = player.inventory[ slots ].topitem.hudimage or nil
			end
		end
	end


end

function hud:draw()
local huddraw
	for huddraw in pairs(hud.live.openwindows) do
		hud.windows[huddraw]:draw(0, 0)
	end

	for menudraw in pairs(hud.live.openmenus) do
		hud.windows[menudraw]:draw(0, 0)
	end

end

function hud:ActionIndex( key, ... )
	-- Next Day button on the hud bar
	if key == "world_AdvanceDay" then world:AdvanceDay() end

	-- for openning and closing windows, used with the close window button on each window as well as the window's button on the hud bar
	if key == "hud_ToggleWindow" and arg[1] then hud:ToggleWindow(arg[1] ) end

	if key == "hud_InventorySlotSelect" and arg[1] then hud:InventorySlotSelect(arg[1]) end
end

-- builds a window if necessary.
function hud:BuildWindow(Type)

--this builds the menu window
	if Type == "Menu" then
		local entries = 0
		local entriestable = {}

		for section, action in ipairs(hud.windows.Inventory.vars.SelectedInventoryItem.actions) do
			entriestable[section] = action
			entries = entries + 1
		end
		local chunkx = 100
		local chunky = entries * 16
		hud.windows.Menu.vars.imagechunks[1] = {x = 0, y = 0, cx = 0, cy = 0, w = chunkx, h = chunky}
		hud.windows.Menu.vars.imagechunks[2] = {x = 295, y = 0, cx = chunkx/2, cy = 0, w = 5, h = chunky}
		hud.windows.Menu.vars.imagechunks[3] = {x = 0, y = 595, cx = 0, cy = chunky/2, w = 100, h = 5}
		hud.windows.Menu.vars.imagechunks[4] = {x = 295, y = 595, cx = chunkx/2, cy = chunky/2, w = 5, h = 5}
		hud.windows.Menu.vars.location = {love.mouse.getPosition()}
		hud.windows.Menu.vars.entries = entriestable
	end
	return {}
end

--gets the coordinates of all of the hud elements currently open to scan to see if the mouse is in them
-- This rebuilds the table hud.live.openwindows

function hud:GetCurrentHudCoords()
HudCoordsTable_S = love.timer.getTime()

	for eachwindow in pairs(hud.live.openwindows) do
		local currentwindowimage = hud.windows[eachwindow]


		--get coordinates for windows
		hud.live.openwindows[eachwindow].coords = {
		currentwindowimage.Position[1],
		currentwindowimage.Position[2],
		currentwindowimage.Position[1] + currentwindowimage.Images[1][3][1]:getWidth(),
		currentwindowimage.Position[2] + currentwindowimage.Images[1][3][1]:getHeight(),
		Buttons = {},
		}

		--get coordinates for buttons
		if currentwindowimage.Buttons[1] then
			for eachbutton in pairs(currentwindowimage.Buttons) do
				local currentbuttonimage = currentwindowimage.Buttons
					hud.live.openwindows[eachwindow].coords.Buttons[eachbutton] = {
					currentwindowimage.Position[1] + currentbuttonimage[eachbutton][1],
					currentwindowimage.Position[2] + currentbuttonimage[eachbutton][2],
					currentwindowimage.Position[1] + currentbuttonimage[eachbutton][1] + currentbuttonimage[eachbutton][4][1]:getWidth(),
					currentwindowimage.Position[2] + currentbuttonimage[eachbutton][2] + currentbuttonimage[eachbutton][4][1]:getHeight(),
					}
			end
		end

		-- get coordinates for grid cells, same thing as a button but no image and not preset. Stored in the button table
		if currentwindowimage.vars.grid then
			hud.live.openwindows[eachwindow].coords.Grid = {}
			for x = 1, currentwindowimage.vars.grid.x do
				hud.live.openwindows[eachwindow].coords.Grid[x] = {}
				for y = 1, currentwindowimage.vars.grid.y do
					hud.live.openwindows[eachwindow].coords.Grid[x][y] = {
						currentwindowimage.Position[1] + currentwindowimage.vars.grid.xorigin + ( (x - 1) * currentwindowimage.vars.grid.size),
						currentwindowimage.Position[2] + currentwindowimage.vars.grid.yorigin + ( (y - 1) * currentwindowimage.vars.grid.size),
						currentwindowimage.Position[1] + currentwindowimage.vars.grid.xorigin + ( x * currentwindowimage.vars.grid.size),
						currentwindowimage.Position[2] + currentwindowimage.vars.grid.yorigin + ( y * currentwindowimage.vars.grid.size),
					}
				end
			end
		end
	end
HudCoordsTable = love.timer.getTime() - HudCoordsTable_S
end

function hud:InventorySlotSelect(Slot) --Button function

-- declares which inventory slot is currently selected
	hud.windows.Inventory.vars.SelectedSlot = Slot

	--update text
	local SlotNames = {head = "Head", torso = "Torso", belt = "Waist", legs = "Legs", feet = "Feet", left_hand = "Left Hand", right_hand = "Right Hand", face_neck = "Face", back = "Back"}
	hud.windows.Inventory.vars.text[1][1] = SlotNames[Slot]
	local SlotWtMult = {legs = 0.75, feet = 0.1, left_hand = 0.35, right_hand = 0.35, face_neck = 0.1}
	local CurSlot = SlotWtMult[Slot] or 1
	hud.windows.Inventory.vars.text[2][1] =
		"Wt: "..tostring(player.inventory[Slot].weight / 1000)..
		"kg / "..tostring(CurSlot * player.strength * player.weight / 1000).."kg"
	local SlotVolMult = {legs = 0.2, feet = 0.05, left_hand = 0.5, right_hand = 0.5, face_neck = 0.1, torso = 0.3, belt = 0.4, back = 1, head = 0.1}
	hud.windows.Inventory.vars.text[3][1] = "Vol: "..tostring(player.inventory[Slot].volume / 1000).."lt / "..tostring(SlotVolMult[Slot] * player.strength * player.volume / 1000).."lt"
end

--[[
	Open Menus. these are like dynamic windows that take their available options as arguments
	only one can be open at any given time
	clicking in them activates an option, otherwise it closes
--]]
function hud:OpenMenu(Type, X, Y, Object)
	--declare the menu's table
	local menu = {"test"}
	menu.x = X
	menu.y = Y


	--add the menu to the hud.live table
	hud.live.openmenu = menu
	-- return the menu table in case it's needed
	return menu
end

function hud:ToggleWindow(window) -- Button function

if window == "Menu" then
	if hud.live.openmenus.Menu then
		hud.live.openmenus.Menu = nil
	else
		hud.live.openmenus.Menu = hud:BuildWindow(window)
	end
else
-- toggles the window by checkign if it's in the openwindows table andthen either adding or removing it as necessary
	if hud.live.openwindows[window] then
		hud.live.openwindows[window] = nil
	else
		hud.live.openwindows[window] = {}
	end

-- updates the coordinates for the hud windows
	hud:GetCurrentHudCoords()
end
end
