map = {
	-- the format for the map files. Some (for testing) use direct modification of map variables, while others
	-- need to be interpreted by a function (a feature to be added later)
	file_format = "direct data",

	--default maps are square using this number to count the number of
	--tiles per side
	size = 200,

	--how many tiles to draw at once. remember to include buffer for
	--smoth scrolling
	display_w = 32,
	display_h = 32,

	--tile buffer in pixels. Should be equal to 1 tile for scrolling
	--and half a tile to make up for the default draw point being attributes
	--the centre of the image
	offset_x = 370,
	offset_y = -250,

	--tile width and height
	tile_w = 64,
	tile_h = 32,

	--scale factor for tiles
	scale_x = 1,
	scale_y = 1,

	--placeholder table for the map
	tile_map = {},

	 -- Set up tile default attributes. This is the tile "prototype"
	tile = {

		-- default dirt attributes
		earth = {
			image = 4, -- [1] = drawn tile
		},
		grass = {
			image = images.world.tiles.grass.meadow_foxtail[1]
		},
	},

	-- Layers, for drawing
	layers = {},

	-- weather for the map
	environment = {
	--[[ my current system for sunlight:
		The sun produces 100 sunlight each day, for the sake of using a percent. Cloud cover is a decimal from
		0.1 to 1 which is a multiplyer to affect the amount of sunlight that reaches the ground (IE 1 means no
		clouds, 0.1 or 0 means ridiculous amounts of clouds). Then trees filter a certain amount of that sunlight,
		also given in a percent of the remaining sunlight, and the rest of the plants below get the rest equally.
	--]]
		sun = 100, 	-- 100 = all of the sun that comes down, this number will change when I have sunlight change
					-- based on the time of year and latitude to factor in the tilt of the planet.
		clouds = 1, 	-- cloud factor to affect sunlight, 1 means no clouds, 0 is wacky solid clouds.
		wind = {0, 0}, -- direction of the prevailing wind. Affects some things like where seeds blow.
		precipitation = false,
		precipitation_type = "snow",
		precipitation_strength = "normal",
	-- temperature modumalation
		macrotemp = world.environment.temperature, -- base temperature for the whole map
		temperature_scope = 20, -- the area around the player most likley to be affected by the temperature_modulate function, in tile radius.
		latitude = 45,
	--TEMPORARY for testing
		geology = "loam",

	},

	--masks for debugging and shit
	masks = {
		thermal = false,
	},
}

function map:load()
	map_x = map_x or 50
	map_y = map_y or 50
	scroll_x = scroll_y or 0
	scroll_y = scroll_y or 0

	for x = 0, self.size do
		map.tile_map[x] = {}
		for y = 0, self.size do
			map.tile_map[x][y] = {
				earth = {
					image = math.random( 1, 4 ),
					surface = "dirt",
					altitude = 0,
					--cover {type, tile to draw, amount increases "rank" at different levels, set somewhere (not yet)}
					cover = {nil, 1, amount = 4},
				},
				plants = {
					grass = {
						species = nil,
						Newgrowth = false,
						Age = 0,
						Level = 0,
					},
					flowers = {
						species = nil,
						Newgrowth = false,
						Age = 0,
						Level = 0,
						SeasonOffset = math.random(1, 14),
						Season = 0,
					},
					trees = {
						species = nil,
						Newgrowth = false,
						Age = 0,
						Level = 0,
						SeasonOffset = math.random(1, 28), -- offsets how the leaves change throughout the year
						Season = 0, -- 0 = spring, 1 = summer, 2 = fall 3 = winter
					},
				},
				objects = {}, -- holds objects
				environment = {
					shadow = 0,
					temperature = math.random(1, 100), --http://mathforum.org/library/drmath/view/56478.html
				},
			}
			setmetatable( map.tile_map[x][y], { __index = map.tile } )
		end
	end
end

function map:update(dt)

	--Move tiles for smooth map scrolling
	if scroll_x < 1 then
		scroll_x = 64 + scroll_x
		if map_x <= self.size then
		map_x = map_x + 1
		map_y = map_y - 1
		end
	end

    if scroll_x > 25 then
		scroll_x = scroll_x - 64
		if map_x >= 0 then
		map_x = map_x - 1
		map_y = map_y + 1
		end
    end

	if scroll_y < 1 then
		scroll_y = scroll_y + 32
		if map_y <= self.size then
		map_x = map_x + 1
		map_y = map_y + 1
		end
	end

    if scroll_y > 25 then
		scroll_y = scroll_y - 32
		if map_y >= 0 then
		map_x = map_x - 1
		map_y = map_y - 1
		end
    end

	-- spread heat around
	map:temperature_modulate()

	--calculate live tile changes, such snow accumulation on tiles
	map:weather()

	--update animations
	if map.environment.precipitation then
		images.world.weather[map.environment.precipitation_type][map.environment.precipitation_strength]:update(dt)
	end
end

function map:draw()
-- every object has a drawing layer. For every tile it goes over every item in that tile and draws it in order of its drawing order.

	--draw dirt. always do this, as grass and other covering still have an
	--alpha channel that need a background that isn't fucking black, fuck
	local PlayerDrawn = false
	for x = 1, self.display_w do
		for y = 1, self.display_h do
			if map.tile_map[y+map_y] and map.tile_map[y+map_y][x + map_x]
			and (x-y) * (map.tile_w/2) + map.offset_x > -2*(map.tile_h) and (x-y) * (map.tile_w/2) + map.offset_x - 64 < 800
			and (x+y)/2*map.tile_h + map.offset_y > -(map.tile_w) and (x+y)/2*map.tile_h + map.offset_y < 600 then

			love.graphics.setColor(255,255,255,255)

			masks:thermal("tint", x, y)

			-- earth
			love.graphics.drawq( images.world.tiles.earth[5],
			quads.tiles.earth.default[1][map.tile_map[y+map_y][x+map_x].earth.image],
            (x-y) * (map.tile_w/2) + map.offset_x + scroll_x,
            (x+y)/2*map.tile_h + map.tile_map[y+map_y][x+map_x].earth.altitude + map.offset_y + scroll_y,
			0,
			map.scale_x,
			map.scale_y,
			1,
			1)

			--temporary, draw earth cover (snow, hurr)
			if map.tile_map[y+map_y][x+map_x].earth.cover[1] then
				love.graphics.drawq( images.world.tiles[map.tile_map[y+map_y][x+map_x].earth.cover[1]],
				quads.tiles.grass.default[1][map.tile_map[y+map_y][x+map_x].earth.cover[2]],
				(x-y) * (map.tile_w/2) + map.offset_x + scroll_x,
				(x+y)/2*map.tile_h + map.tile_map[y+map_y][x+map_x].earth.altitude + map.offset_y + scroll_y,
				0,
				map.scale_x,
				map.scale_y,
				1,
				1)
			end

			map:objects_draw(x, y)

			-- player
			-- Check if the player has been drawn and draw him if not
			if y == 30 and PlayerDrawn == false then
				player:draw(map.tile_map[y+map_y][x+map_x].earth.altitude)
				PlayerDrawn = true
			end

			masks:thermal("type", x, y)
			end
		end
	end--]]

	 masks.precipitation(map.environment.precipitation_type)

end

-- adds objects to map.layers

function map:add_image(layer_table, object)
	local pos
	local object
	return 1
end

-- builds a table that stores all of the images on the map to be drawn, in seperate layers.
function map:build_layers()
	local layer_table = {}

	for x = 1, map.size do
		for y = 1, map.size do
			if map.tile_map[x][y].objects[1] then
				for i, v in ipairs(map.tile_map[x][y].objects) do
				local OBJECT =  map.tile_map[x][y].objects[i]
				table.insert( layer_table, map:add_image(OBJECT) )

				end
			end
		end
	end

	return layer_table
end

function map:getdaylength()
	local L_var = map.environment.latitude
	local J_var = world.time.Day_total
	local P_var = math.asin(
		0.39795 * math.cos(
			0.2163108 + 2 * math.atan(
				0.9671396 * math.tan(
					0.00860 * (J_var - 186)
				)
			)
		)
	)

	local D_var = 24 - ( 24 / math.pi) * math.acos(
		( math.sin( 0.8333 * math.pi / 180 ) + math.sin( L_var * math.pi / 180 ) * math.sin( P_var ) )
		/
		( math.cos( L_var * math.pi / 180 ) * math.cos( P_var ) )
	)

	return D_var
end

function map:newday()

	local x
	local y

	-- take actions for each object (could replace the next function)
	for x = 1, self.size do
		for y = 1, self.size do
			for obj_i, obj_v in pairs(map.tile_map[x][y].objects) do
				objects:Act(obj_i, obj_v, x, y)
			end
		end
	end


	--Take actions for each plant
	for x = 1, self.size do
		for y = 1, self.size do
			local SunExposure = map.tile_map[x][y].environment.shadow
		end
	end
--[[

-- anything else do be done afterwards
	for x = 1, self.size do
		for y = 1, self.size do

		-- end growth clean up
			map.tile_map[x][y].plants.grass.Newgrowth = false
			map.tile_map[x][y].plants.flowers.Newgrowth = false
			map.tile_map[x][y].plants.trees.Newgrowth = false
		-- Update effective tree season for drawing leaves
			local TreeTable = map.tile_map[x][y].plants.trees
			if world.time.month < 3 then TreeTable.Season = 3 end
			if world.time.month == 3 and world.time.day > TreeTable.SeasonOffset then TreeTable.Season = 0 end
			if world.time.month == 6 and world.time.day > TreeTable.SeasonOffset then TreeTable.Season = 1 end
			if world.time.month == 9 and world.time.day > TreeTable.SeasonOffset then TreeTable.Season = 2 end
			if world.time.month == 12 and world.time.day > TreeTable.SeasonOffset then TreeTable.Season = 3 end

		-- Update effective Flower season for bloomin and seeding
			local FlowerTable = map.tile_map[x][y].plants.flowers
			if world.time.month < 2 then FlowerTable.Season = 4 end
			if world.time.month == 2 and world.time.day > FlowerTable.SeasonOffset then FlowerTable.Season = 0 end
			if world.time.month == 4 and world.time.day > FlowerTable.SeasonOffset then FlowerTable.Season = 1 end
			if world.time.month == 6 and world.time.day > FlowerTable.SeasonOffset then FlowerTable.Season = 2 end
			if world.time.month == 8 and world.time.day > FlowerTable.SeasonOffset then FlowerTable.Season = 3 end
			if world.time.month == 11 and world.time.day > FlowerTable.SeasonOffset then FlowerTable.Season = 4 end
		end
	end
--]]

-- do daily actions and checks for each object

-- Set new weather and get results from the day's weather. Doesn't do anythign yet, this is just a placeholder.
	local SunExposure = map.environment.sun - map.environment.clouds
	map.environment.sun = map.environment.sun -- to add variance later on based on time of year and latitude
	map.environment.clouds = map.environment.clouds -- once I have proper weather effects in, this will vary.

-- Reset shadows before getting new ones
	for x = 1, self.size do
		for y = 1, self.size do
			map.tile_map[x][y].environment.shadow = 0

		end
	end

-- get new shadows for each tile
	for x = 1, self.size do
		for y = 1, self.size do
			local TreeTable = map.tile_map[x][y].plants.trees
			if TreeTable.species then
				TreeTable.species:SetNewShadows(x, y, TreeTable.Level)
			end
		end
	end

end

--for drawing object sin the map
function map:objects_draw(x, y)
	if map.tile_map[y+map_y][x + map_x].objects[1] then
		for i, v in ipairs(map.tile_map[y+map_y][x + map_x].objects) do
			local OBJECT =  map.tile_map[y+map_y][x+map_x].objects[i]
			if map.tile_map[y+map_y][x + map_x].objects[i].IMAGE_TYPE then

				if OBJECT.IMAGE_TYPE >= 20 then

					for subimages = 1, (OBJECT.IMAGE_TYPE - 19) do
						local QUAD = quads.tiles[ OBJECT.base_name ].default
						love.graphics.drawq(
						OBJECT.worldimage,
						QUAD[1][ OBJECT.age_step ],
						(x-y) * (map.tile_w/2) + map.offset_x + scroll_x + QUAD.x_offset,
						(x+y)/2*map.tile_h + map.tile_map[y+map_y][x+map_x].earth.altitude + map.offset_y + scroll_y + QUAD.y_offset,
						0,
						map.scale_x,
						map.scale_y,
						1,
						1)
					end
				end
			else
			love.graphics.draw( OBJECT.worldimage,
			(x-y) * (map.tile_w/2) + map.offset_x + scroll_x,
			(x+y)/2*map.tile_h + map.tile_map[y+map_y][x+map_x].earth.altitude + map.offset_y + scroll_y
			)
			end
		end
	end
end

function map:temperature_modulate()

	--pick a random tile within a certain radius of the player
	local xlow = map_x - map.environment.temperature_scope + 15
	local xhigh = map_x + map.environment.temperature_scope + 15
	local ylow = map_y - map.environment.temperature_scope + 15
	local yhigh = map_y + map.environment.temperature_scope + 15

	local rdm_modulate_x = math.random(xlow, xhigh)
	local rdm_modulate_y = math.random(ylow, yhigh)

	if rdm_modulate_x - 1 < 2 then return end
	if rdm_modulate_x + 1 > 99 then return end
	if rdm_modulate_y - 1 < 2 then return end
	if rdm_modulate_y + 1 > 99 then return end

	-- set variables to affected tiles (selected tiles and orthagonally adjacent tiles)
	local centre = map.tile_map[rdm_modulate_x][rdm_modulate_y].environment.temperature
	local left_ort = map.tile_map[rdm_modulate_x - 1][rdm_modulate_y].environment.temperature
	local right_ort = map.tile_map[rdm_modulate_x + 1][rdm_modulate_y].environment.temperature
	local top_ort = map.tile_map[rdm_modulate_x][rdm_modulate_y - 1].environment.temperature
	local bottom_ort = map.tile_map[rdm_modulate_x][rdm_modulate_y + 1].environment.temperature

	-- get temperature differences for each adjacent tile
	local left_mod = math.floor((centre - left_ort)/2)
	local right_mod = math.floor((centre - right_ort)/2)
	local top_mod = math.floor((centre - top_ort)/2)
	local bottom_mod = math.floor((centre - bottom_ort)/2)

	--Change temepratures
	map.tile_map[rdm_modulate_x - 1][rdm_modulate_y].environment.temperature = left_ort + left_mod
	map.tile_map[rdm_modulate_x + 1][rdm_modulate_y].environment.temperature = right_ort + right_mod
	map.tile_map[rdm_modulate_x][rdm_modulate_y - 1].environment.temperature = top_ort + top_mod
	map.tile_map[rdm_modulate_x][rdm_modulate_y + 1].environment.temperature = bottom_ort + bottom_mod
	map.tile_map[rdm_modulate_x][rdm_modulate_y].environment.temperature = centre + -1*(left_mod + right_mod + top_mod + bottom_mod)
end

function map:weather()
	if map.environment.precipitation then
		if map.environment.precipitation_type == "snow" then
			local temp_x = math.random(1, map.size)
			local temp_y = math.random(1, map.size)
			map.tile_map[temp_x][temp_y].earth.cover.amount = map.tile_map[temp_x][temp_y].earth.cover.amount + 1 -- change to snowfall heaviness number
			if map.tile_map[temp_x][temp_y].earth.cover.amount > 0 then
				map.tile_map[temp_x][temp_y].earth.cover[1] = "snow"
				map.tile_map[temp_x][temp_y].earth.cover[2] = 2
				if map.tile_map[temp_x][temp_y].earth.cover.amount < 2 then
					map.tile_map[temp_x][temp_y].earth.cover[2] = 1
				elseif map.tile_map[temp_x][temp_y].earth.cover.amount < 3 then
					map.tile_map[temp_x][temp_y].earth.cover[2] = 2
				elseif map.tile_map[temp_x][temp_y].earth.cover.amount < 4 then
					map.tile_map[temp_x][temp_y].earth.cover[2] = 3
				elseif map.tile_map[temp_x][temp_y].earth.cover.amount< 5 then
					map.tile_map[temp_x][temp_y].earth.cover[2] = 4
				end
			end
		end
	end
end
